/* valaclassregisterfunction.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valaclassregisterfunction.h"
#include <ccode/valaccodedeclaration.h>
#include <ccode/valaccodemodifiers.h>
#include <ccode/valaccodevariabledeclarator.h>
#include <ccode/valaccodeconstant.h>
#include <ccode/valaccodeexpression.h>
#include <ccode/valaccodedeclarator.h>
#include <ccode/valaccodenode.h>
#include <ccode/valaccodefunctioncall.h>
#include <ccode/valaccodeidentifier.h>
#include <ccode/valaccodeexpressionstatement.h>
#include <vala/valatypereference.h>
#include <vala/valainterface.h>

struct _ValaClassRegisterFunctionPrivate {
	ValaClass* _class_reference;
};
#define VALA_CLASS_REGISTER_FUNCTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CLASS_REGISTER_FUNCTION, ValaClassRegisterFunctionPrivate))
enum  {
	VALA_CLASS_REGISTER_FUNCTION_DUMMY_PROPERTY,
	VALA_CLASS_REGISTER_FUNCTION_CLASS_REFERENCE,
};
static ValaDataType* vala_class_register_function_real_get_type_declaration (ValaTypeRegisterFunction* base);
static char* vala_class_register_function_real_get_type_struct_name (ValaTypeRegisterFunction* base);
static char* vala_class_register_function_real_get_class_init_func_name (ValaTypeRegisterFunction* base);
static char* vala_class_register_function_real_get_instance_struct_size (ValaTypeRegisterFunction* base);
static char* vala_class_register_function_real_get_instance_init_func_name (ValaTypeRegisterFunction* base);
static char* vala_class_register_function_real_get_parent_type_name (ValaTypeRegisterFunction* base);
static char* vala_class_register_function_real_get_type_flags (ValaTypeRegisterFunction* base);
static ValaCCodeFragment* vala_class_register_function_real_get_type_interface_init_statements (ValaTypeRegisterFunction* base);
static gpointer vala_class_register_function_parent_class = NULL;
static void vala_class_register_function_dispose (GObject * obj);


/**
 * Creates a new C function to register the specified class at runtime.
 *
 * @param cl a class
 * @return   newly created class register function
 */
ValaClassRegisterFunction* vala_class_register_function_new (ValaClass* cl)
{
	g_return_val_if_fail (VALA_IS_CLASS (cl), NULL);
	GParameter * __params = g_new0 (GParameter, 1);
	GParameter * __params_it = __params;
	(__params_it->name = "class-reference", g_value_init (&__params_it->value, VALA_TYPE_CLASS), g_value_set_object (&__params_it->value, cl), __params_it++);
	ValaClassRegisterFunction * self = g_object_newv (VALA_TYPE_CLASS_REGISTER_FUNCTION, __params_it - __params, __params);
	return self;
}


static ValaDataType* vala_class_register_function_real_get_type_declaration (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (base);
	return VALA_DATA_TYPE (vala_class_register_function_get_class_reference (self));
}


static char* vala_class_register_function_real_get_type_struct_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (base);
	return g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (vala_class_register_function_get_class_reference (self)), FALSE));
}


static char* vala_class_register_function_real_get_class_init_func_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (base);
	char* __temp2 = NULL;
	char* __temp3 = NULL;
	return (__temp3 = g_strdup_printf ("%s_class_init", (__temp2 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (vala_class_register_function_get_class_reference (self)), NULL))), (__temp2 == NULL ? NULL : (__temp2 = (g_free (__temp2), NULL))), __temp3);
}


static char* vala_class_register_function_real_get_instance_struct_size (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (base);
	return g_strdup_printf ("sizeof (%s)", vala_data_type_get_cname (VALA_DATA_TYPE (vala_class_register_function_get_class_reference (self)), FALSE));
}


static char* vala_class_register_function_real_get_instance_init_func_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (base);
	char* __temp6 = NULL;
	char* __temp7 = NULL;
	return (__temp7 = g_strdup_printf ("%s_init", (__temp6 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (vala_class_register_function_get_class_reference (self)), NULL))), (__temp6 == NULL ? NULL : (__temp6 = (g_free (__temp6), NULL))), __temp7);
}


static char* vala_class_register_function_real_get_parent_type_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (base);
	return vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (vala_class_get_base_class (vala_class_register_function_get_class_reference (self))), "TYPE_");
}


static char* vala_class_register_function_real_get_type_flags (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (base);
	if (vala_class_get_is_abstract (vala_class_register_function_get_class_reference (self))) {
		return "G_TYPE_FLAG_ABSTRACT";
	} else {
		return "0";
	}
}


static ValaCCodeFragment* vala_class_register_function_real_get_type_interface_init_statements (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (base);
	ValaCCodeFragment* frag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	{
		GList* __temp21 = NULL;
		__temp21 = vala_class_get_base_types (vala_class_register_function_get_class_reference (self));
		GList* base_type_it;
		for (base_type_it = __temp21; base_type_it != NULL; base_type_it = base_type_it->next) {
			ValaTypeReference* base_type = base_type_it->data;
			{
				if (!(VALA_IS_INTERFACE (vala_type_reference_get_data_type (base_type)))) {
					continue;
				}
				ValaInterface* iface = VALA_INTERFACE (vala_type_reference_get_data_type (base_type));
				char* __temp12 = NULL;
				char* __temp13 = NULL;
				char* iface_info_name = (__temp13 = g_strdup_printf ("%s_info", (__temp12 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (iface), NULL))), (__temp12 == NULL ? NULL : (__temp12 = (g_free (__temp12), NULL))), __temp13);
				ValaCCodeDeclaration* ctypedecl = vala_ccode_declaration_new ("const GInterfaceInfo");
				vala_ccode_declaration_set_modifiers (ctypedecl, VALA_CCODE_MODIFIERS_STATIC);
				ValaCCodeVariableDeclarator* __temp16 = NULL;
				char* __temp15 = NULL;
				char* __temp14 = NULL;
				vala_ccode_declaration_add_declarator (ctypedecl, VALA_CCODE_DECLARATOR ((__temp16 = vala_ccode_variable_declarator_new_with_initializer (iface_info_name, VALA_CCODE_EXPRESSION (vala_ccode_constant_new (g_strdup_printf ("{ (GInterfaceInitFunc) %s_%s_interface_init, (GInterfaceFinalizeFunc) NULL, NULL}", (__temp14 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (vala_class_register_function_get_class_reference (self)), NULL)), (__temp15 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (iface), NULL)))))))));
				(__temp16 == NULL ? NULL : (__temp16 = (g_object_unref (__temp16), NULL)));
				(__temp15 == NULL ? NULL : (__temp15 = (g_free (__temp15), NULL)));
				(__temp14 == NULL ? NULL : (__temp14 = (g_free (__temp14), NULL)));
				vala_ccode_fragment_append (frag, VALA_CCODE_NODE (ctypedecl));
				ValaCCodeFunctionCall* reg_call = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_add_interface_static")));
				ValaCCodeIdentifier* __temp17 = NULL;
				vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp17 = vala_ccode_identifier_new ("g_define_type_id"))));
				(__temp17 == NULL ? NULL : (__temp17 = (g_object_unref (__temp17), NULL)));
				ValaCCodeIdentifier* __temp18 = NULL;
				vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp18 = vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_")))));
				(__temp18 == NULL ? NULL : (__temp18 = (g_object_unref (__temp18), NULL)));
				ValaCCodeIdentifier* __temp19 = NULL;
				vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp19 = vala_ccode_identifier_new (g_strdup_printf ("&%s", iface_info_name)))));
				(__temp19 == NULL ? NULL : (__temp19 = (g_object_unref (__temp19), NULL)));
				ValaCCodeExpressionStatement* __temp20 = NULL;
				vala_ccode_fragment_append (frag, VALA_CCODE_NODE ((__temp20 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (reg_call)))));
				(__temp20 == NULL ? NULL : (__temp20 = (g_object_unref (__temp20), NULL)));
				(iface_info_name == NULL ? NULL : (iface_info_name = (g_free (iface_info_name), NULL)));
				(ctypedecl == NULL ? NULL : (ctypedecl = (g_object_unref (ctypedecl), NULL)));
				(reg_call == NULL ? NULL : (reg_call = (g_object_unref (reg_call), NULL)));
			}
		}
		(__temp21 == NULL ? NULL : (__temp21 = (g_list_free (__temp21), NULL)));
	}
	return frag;
	(frag == NULL ? NULL : (frag = (g_object_unref (frag), NULL)));
}


ValaClass* vala_class_register_function_get_class_reference (ValaClassRegisterFunction* self)
{
	g_return_val_if_fail (VALA_IS_CLASS_REGISTER_FUNCTION (self), NULL);
	return self->priv->_class_reference;
}


void vala_class_register_function_set_class_reference (ValaClassRegisterFunction* self, ValaClass* value)
{
	g_return_if_fail (VALA_IS_CLASS_REGISTER_FUNCTION (self));
	ValaClass* __temp24 = NULL;
	self->priv->_class_reference = (__temp24 = g_object_ref (value), (self->priv->_class_reference == NULL ? NULL : (self->priv->_class_reference = (g_object_unref (self->priv->_class_reference), NULL))), __temp24);
}


static void vala_class_register_function_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (object);
	switch (property_id) {
		case VALA_CLASS_REGISTER_FUNCTION_CLASS_REFERENCE:
		g_value_set_object (value, vala_class_register_function_get_class_reference (self));
		break;
	}
}


static void vala_class_register_function_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (object);
	switch (property_id) {
		case VALA_CLASS_REGISTER_FUNCTION_CLASS_REFERENCE:
		vala_class_register_function_set_class_reference (self, g_value_get_object (value));
		break;
	}
}


static void vala_class_register_function_class_init (ValaClassRegisterFunctionClass * klass)
{
	vala_class_register_function_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaClassRegisterFunctionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_class_register_function_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_class_register_function_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_class_register_function_dispose;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_type_declaration = vala_class_register_function_real_get_type_declaration;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_type_struct_name = vala_class_register_function_real_get_type_struct_name;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_class_init_func_name = vala_class_register_function_real_get_class_init_func_name;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_instance_struct_size = vala_class_register_function_real_get_instance_struct_size;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_instance_init_func_name = vala_class_register_function_real_get_instance_init_func_name;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_parent_type_name = vala_class_register_function_real_get_parent_type_name;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_type_flags = vala_class_register_function_real_get_type_flags;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_type_interface_init_statements = vala_class_register_function_real_get_type_interface_init_statements;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CLASS_REGISTER_FUNCTION_CLASS_REFERENCE, g_param_spec_object ("class-reference", "foo", "bar", VALA_TYPE_CLASS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_class_register_function_init (ValaClassRegisterFunction * self)
{
	self->priv = VALA_CLASS_REGISTER_FUNCTION_GET_PRIVATE (self);
}


static void vala_class_register_function_dispose (GObject * obj)
{
	ValaClassRegisterFunction * self = VALA_CLASS_REGISTER_FUNCTION (obj);
	(self->priv->_class_reference == NULL ? NULL : (self->priv->_class_reference = (g_object_unref (self->priv->_class_reference), NULL)));
	ValaClassRegisterFunctionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CLASS_REGISTER_FUNCTION_CLASS (g_type_class_peek (VALA_TYPE_CLASS_REGISTER_FUNCTION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_class_register_function_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaClassRegisterFunctionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_class_register_function_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaClassRegisterFunction), 0, (GInstanceInitFunc) vala_class_register_function_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_TYPE_REGISTER_FUNCTION, "ValaClassRegisterFunction", &g_define_type_info, 0);
	}
	return g_define_type_id;
}




