/* valatypeparameter.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valatypeparameter.h"
#include <vala/valasourcereference.h>
#include <vala/valatypeparameter.h>
#include <vala/valacodevisitor.h>

struct _ValaTypeParameterPrivate {
	char* _name;
};
#define VALA_TYPE_PARAMETER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_TYPE_PARAMETER, ValaTypeParameterPrivate))
enum  {
	VALA_TYPE_PARAMETER_DUMMY_PROPERTY,
	VALA_TYPE_PARAMETER_NAME,
};
static void vala_type_parameter_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static gpointer vala_type_parameter_parent_class = NULL;
static void vala_type_parameter_dispose (GObject * obj);


/**
 * Creates a new generic type parameter.
 *
 * @param name   parameter name
 * @param source reference to source code
 * @return       newly created generic type parameter
 */
ValaTypeParameter* vala_type_parameter_new (char* _name, ValaSourceReference* source)
{
	g_return_val_if_fail (_name != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	GParameter * __params = g_new0 (GParameter, 2);
	GParameter * __params_it = __params;
	(__params_it->name = "name", g_value_init (&__params_it->value, G_TYPE_STRING), g_value_set_string (&__params_it->value, _name), __params_it++);
	(__params_it->name = "source-reference", g_value_init (&__params_it->value, VALA_TYPE_SOURCE_REFERENCE), g_value_set_object (&__params_it->value, source), __params_it++);
	ValaTypeParameter * self = g_object_newv (VALA_TYPE_TYPE_PARAMETER, __params_it - __params, __params);
	return self;
}


static void vala_type_parameter_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaTypeParameter * self = VALA_TYPE_PARAMETER (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_type_parameter (visitor, self);
}


char* vala_type_parameter_get_name (ValaTypeParameter* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_PARAMETER (self), NULL);
	return self->priv->_name;
}


void vala_type_parameter_set_name (ValaTypeParameter* self, const char* value)
{
	g_return_if_fail (VALA_IS_TYPE_PARAMETER (self));
	char* __temp1 = NULL;
	self->priv->_name = (__temp1 = g_strdup (value), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp1);
}


static void vala_type_parameter_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaTypeParameter * self = VALA_TYPE_PARAMETER (object);
	switch (property_id) {
		case VALA_TYPE_PARAMETER_NAME:
		g_value_set_string (value, vala_type_parameter_get_name (self));
		break;
	}
}


static void vala_type_parameter_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaTypeParameter * self = VALA_TYPE_PARAMETER (object);
	switch (property_id) {
		case VALA_TYPE_PARAMETER_NAME:
		vala_type_parameter_set_name (self, g_value_get_string (value));
		break;
	}
}


static void vala_type_parameter_class_init (ValaTypeParameterClass * klass)
{
	vala_type_parameter_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaTypeParameterPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_type_parameter_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_type_parameter_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_type_parameter_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_type_parameter_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_PARAMETER_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_type_parameter_init (ValaTypeParameter * self)
{
	self->priv = VALA_TYPE_PARAMETER_GET_PRIVATE (self);
}


static void vala_type_parameter_dispose (GObject * obj)
{
	ValaTypeParameter * self = VALA_TYPE_PARAMETER (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	ValaTypeParameterClass * klass;
	GObjectClass * parent_class;
	klass = VALA_TYPE_PARAMETER_CLASS (g_type_class_peek (VALA_TYPE_TYPE_PARAMETER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_type_parameter_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaTypeParameterClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_type_parameter_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaTypeParameter), 0, (GInstanceInitFunc) vala_type_parameter_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaTypeParameter", &g_define_type_info, 0);
	}
	return g_define_type_id;
}




